module TicketflyPlus
    class Objects
        class Base

            private

            def get_param_info(event_param_items, param_class)
                arr = Array.new
                if !event_param_items.empty?
                    event_param_items.each {|item| arr.push param_class.new(item)}
                end
                arr
            end
        end

        class Event < Base
            attr_accessor :additionalInfo
            attr_accessor :additionalTicketText
            attr_accessor :ageLimit
            attr_accessor :ageLimitCode
            attr_accessor :dateCreated 
            attr_accessor :doorsDate 
            attr_accessor :endDate 
            attr_accessor :eventStatus
            attr_accessor :eventStatusCode
            attr_accessor :eventStatusMessage
            attr_accessor :externalTicketingUrls 
            attr_accessor :facebookEventId
            attr_accessor :facebookEventIdString
            attr_accessor :featured
            attr_accessor :headliners 
            attr_accessor :headlinersName
            attr_accessor :id
            attr_accessor :image 
            attr_accessor :lastUpdated
            attr_accessor :name
            attr_accessor :offSaleDate 
            attr_accessor :onSaleDate 
            attr_accessor :onSaleDates 
            attr_accessor :org  
            attr_accessor :promoterName
            attr_accessor :publishDate
            attr_accessor :published
            attr_accessor :purchaseSkin
            attr_accessor :showType
            attr_accessor :showTypeCode
            attr_accessor :slug
            attr_accessor :sponsorImage
            attr_accessor :sponsorName
            attr_accessor :startDate 
            attr_accessor :supports 
            attr_accessor :supportsName
            attr_accessor :ticketPrice
            attr_accessor :ticketPurchaseUrl
            attr_accessor :topLineInfo
            attr_accessor :urlEventDetailsUrl
            attr_accessor :venue 

            def initialize(event)
                @additionalInfo = event['additionalInfo']
                @additionalTicketText = event['additionalTicketText']
                @ageLimit = event['ageLimit']
                @ageLimitCode = event['ageLimitCode']
                @dateCreated = !event['dateCreated'].nil? ? DateTime.parse(event['dateCreated']) : event['dateCreated']
                @doorsDate = !event['doorsDate'].nil? ? DateTime.parse(event['doorsDate']) : event['doorsDate']
                @endDate = !event['endDate'].nil? ? DateTime.parse(event['endDate']) : event['endDate']
                @eventStatus = event['eventStatus']
                @eventStatusCode = event['eventStatusCode']
                @eventStatusMessage = event['eventStatusMessage']
                @externalTicketingUrls = !event['externalTicketingUrls'].nil? ? get_param_info(event['externalTicketingUrls'], Objects::TicketingUrl ) : event['externalTicketingUrls']
                @facebookEventId = event['facebookEventId']
                @facebookEventIdString = event['facebookEventIdString']
                @featured = event['featured']
                @headliners = !event['headliners'].nil? ? get_param_info(event['headliners'], Objects::Attraction) : event['headliners']
                @headlinersName = event['headlinersName']
                @id = event['id']
                @image = !event['image'].nil? ? Objects::Image.new(event['image']) : event['image']
                @lastUpdated = !event['lastUpdated'].nil? ? DateTime.parse(event['lastUpdated']) : event['lastUpdated']
                @name = event['name']
                @offSaleDate = !event['offSaleDate'].nil? ? DateTime.parse(event['offSaleDate']) : event['offSaleDate']
                @onSaleDate = !event['onSaleDate'].nil? ? DateTime.parse(event['onSaleDate']) : event['onSaleDate']
                @onSaleDates = !event['onSaleDates'].nil? ? get_param_info(event['onSaleDates'], Objects::SaleDate) : event['onSaleDates']
                @org = !event['org'].nil? ? Objects::Org.new(event['org']) : event['org'] 
                @promoterName = event['promoterName']
                @publishDate = !event['publishDate'].nil? ? DateTime.parse(event['publishDate']) : event['publishDate']
                @published = event['published']
                @purchaseSkin = !event['purchaseSkin'].nil? ? Objects::PurchaseSkin.new(event['purchaseSkin']) : event['purchaseSkin']
                @showType = event['showType']
                @showTypeCode = event['showTypeCode']
                @slug = event['slug']
                @sponsorImage = !event['sponsorImage'].nil? ? Objects::Image.new(event['sponsorImage']) : event['sponsorImage']
                @sponsorName = event['sponsorName']
                @startDate = !event['startDate'].nil? ? DateTime.parse(event['startDate']) : event['startDate']
                @supports = !event['supports'].nil? ? get_param_info(event['supports'], Objects::Attraction) : event['supports']
                @supportsName = event['supportsName']
                @ticketPrice = event['ticketPrice']
                @ticketPurchaseUrl = event['ticketPurchaseUrl']
                @topLineInfo = event['topLineInfo']
                @urlEventDetailsUrl = event['urlEventDetailsUrl']
                @venue = !event['venue'].nil? ? Objects::Venue.new(event['venue']) : event['venue']
            end	

            def share_a_sale(shareasale)
                    s_url = 'http://www.shareasale.com/r.cfm?u=' + shareasale.to_s + '&b=234786&m=27601&afftrack=&urllink='
                    share_a_sale_url = s_url + @ticketPurchaseUrl.
                                    gsub('&', '%26').
                                    gsub('.', '%2E').
                                    gsub('/', '%2F').
                                    gsub('=', '%3D').
                                    gsub('?', '%3F').
                                    gsub('_', '%5F')
                    return share_a_sale_url
            end
        end


        class Org
            attr_accessor :id
            attr_accessor :name
            attr_accessor :promoter
            attr_accessor :timeZone 

            def initialize(org)
                @id = org['id']
                @name = org['name']
                @promoter = org['promoter']
                @timeZone = org['timeZone']
            end
        end

        class Venue
            attr_accessor :address1
            attr_accessor :address2
            attr_accessor :blurb
            attr_accessor :city
            attr_accessor :country
            attr_accessor :id
            attr_accessor :image 
            attr_accessor :lat
            attr_accessor :lng
            attr_accessor :metroCode
            attr_accessor :name
            attr_accessor :postalCode
            attr_accessor :stateProvince
            attr_accessor :timeZone 
            attr_accessor :url
            attr_accessor :urlFacebook
            attr_accessor :urlTwitter

            def initialize(venue)
                @address1 = venue['address1']
                @address2 = venue['address2']
                @blurb = venue['blurb']
                @city = venue['city']
                @country = venue['country']
                @id = venue['id']
                @image = !venue['image'].nil? ? Objects::Image.new(venue['image']) : venue['image'] 
                @lat = venue['lat']
                @lng = venue['lng']
                @metroCode = venue['metrocode']
                @name = venue['name']
                @postalCode = venue['postalCode']
                @stateProvince = venue['stateProvince']
                @timeZone = venue['timeZone'] 
                @url = venue['url']
                @urlFacebook = venue['urlFacebook']
                @urlTwitter = venue['urlTwitter']
            end
        end

        class YoutubeVideo
            attr_accessor :embedCodeIframe
            attr_accessor :title
            attr_accessor :videoId

            def initialize(video)
                @embedcodeiframe = video['embedcodeiframe']
                @title = video['title']
                @videoid = video['videoid']
            end
        end

        class Attraction < Base
            attr_accessor :embedAudio
            attr_accessor :embedVideo
            attr_accessor :eventDescription
            attr_accessor :id
            attr_accessor :image 
            attr_accessor :isMobileFriendlyBoolean
            attr_accessor :name
            attr_accessor :startTime 
            attr_accessor :twitterScreenName
            attr_accessor :urlFacebook
            attr_accessor :urlMySpace
            attr_accessor :urlOfficialWebsite
            attr_accessor :urlPurchaseMusic
            attr_accessor :urlTwitter
            attr_accessor :youtubeVideos 

            def initialize(attraction)
                @embedAudio = attraction['embedAudio']
                @embedVideo = attraction['embedVideo']
                @eventDescription = attraction['eventDescription']
                @id = attraction['id']
                @image = !attraction['image'].nil? ? Objects::Image.new(attraction['image']) : attraction['image']
                @isMobileFriendlyBoolean = attraction['isMobileFriendlyBoolean']
                @name = attraction['name']
                @startTime = !attraction['startTime'].nil? ? Time.parse(attraction['startTime']) : attraction['startTime'] 
                @twitterScreenName = attraction['twitterScreenName']
                @urlFacebook = attraction['urlFacebook']
                @urlMySpace = attraction['urlMySpace']
                @urlOfficialWebsite = attraction['urlOfficialWebsite']
                @urlPurchaseMusic = attraction['urlPurchaseMusic']
                @urlTwitter = attraction['urlTwitter']
                @youtubeVideos = !attraction['youtubeVideos'].nil? ? get_param_info(attraction['youtubeVideos'], Objects::YoutubeVideo) : attraction['youtubeVideos']
            end
        end

        class ImageDetails
            attr_accessor :height
            attr_accessor :path
            attr_accessor :width

            def initialize(image_details)
                @height = image_details['height']
                @path = image_details['path']
                @width = image_details['width']
            end
        end

        class Image
            attr_accessor :jumbo
            attr_accessor :jumbo1
            attr_accessor :large
            attr_accessor :large1
            attr_accessor :medium
            attr_accessor :medium1
            attr_accessor :original
            attr_accessor :small
            attr_accessor :small1
            attr_accessor :square
            attr_accessor :squareSmall
            attr_accessor :xlarge
            attr_accessor :xlarge1
            attr_accessor :xxlarge
            attr_accessor :xxlarge1

            def initialize(image)
                @jumbo = check_nil(image['jumbo'])
                @jumbo1 = check_nil(image['jumbo1'])
                @large = check_nil(image['large'])
                @large1 = check_nil(image['large1'])
                @medium = check_nil(image['medium'])
                @medium1 = check_nil(image['medium1'])
                @original = check_nil(image['original'])
                @small = check_nil(image['small'])
                @small1 = check_nil(image['small1'])
                @square = check_nil(image['square'])
                @squareSmall = check_nil(image['squareSmall'])
                @xlarge = check_nil(image['xlarge'])
                @xlarge1 = check_nil(image['xlarge1'])
                @xxlarge = check_nil(image['xxlarge'])
                @xxlarge1 = check_nil(image['xxlarge1'])
            end

            private

            def check_nil(image)
                !image.nil? ? Objects::ImageDetails.new(image) : image
            end
        end

        class TicketingUrl
            attr_accessor :name
            attr_accessor :url

            def initialize(url)
                @name = url['name']
                @url = url['url']
            end
        end

        class SaleDate
            attr_accessor :endDate
            attr_accessor :name
            attr_accessor :startDate

            def initialize(date)
                @endDate = date['endDate'] != nil ? DateTime.parse(date['endDate']) : date['endDate']
                @name = date['name']
                @startDate = date['startDate'] != nil ? DateTime.parse(date['startDate']) : date['startDate']
            end
        end

        class PurchaseSkin
            attr_accessor :backgroundColor
            attr_accessor :backgroundFixed
            attr_accessor :backgroundImage
            attr_accessor :backgroundPositionType
            attr_accessor :backgroundRepeatType
            attr_accessor :bodyHoverColor
            attr_accessor :bodyLinkColor
            attr_accessor :bodyTextColor
            attr_accessor :class
            attr_accessor :contentBackgroundColor
            attr_accessor :dateCreated
            attr_accessor :eventInfoBoxBackgroundColor
            attr_accessor :eventInfoBoxBorderColor
            attr_accessor :eventInfoBoxHeadlinerColor
            attr_accessor :eventInfoBoxHoverColor
            attr_accessor :eventInfoBoxLinkColor
            attr_accessor :eventInfoBoxTextColor
            attr_accessor :eventInfoSubheadColor
            attr_accessor :eventInfoSubheadTextColor
            attr_accessor :footerHtml
            attr_accessor :footerImage
            attr_accessor :footerTextColor
            attr_accessor :footerTicketflyLogo
            attr_accessor :headerHtml
            attr_accessor :headerImage
            attr_accessor :headerTextColor
            attr_accessor :headerTicketflyLogo
            attr_accessor :id
            attr_accessor :lastUpdated
            attr_accessor :name
            attr_accessor :org
            attr_accessor :secondaryBorderColor
            attr_accessor :theme
            attr_accessor :ticketButtonColor
            attr_accessor :ticketButtonTextColor

            def initialize(skin)
                @backgroundColor = skin['backgroundColor']
                @backgroundFixed = skin['backgroundFixed']
                @backgroundImage = skin['backgroundImage']
                @backgroundPositionType = skin['backgroundPositionType']
                @backgroundRepeatType = skin['backgroundRepeatType']
                @bodyHoverColor = skin['bodyHoverColor']
                @bodyLinkColor = skin['bodyLinkColor']
                @bodyTextColor = skin['bodyTextColor']
                @class = skin['class']
                @contentBackgroundColor = skin['contentBackgroundColor']
                @dateCreated = skin['dateCreated']
                @eventInfoBoxBackgroundColor = skin['eventInfoBoxBackgroundColor']
                @eventInfoBoxBorderColor = skin['eventInfoBoxBorderColor']
                @eventInfoBoxHeadlinerColor = skin['eventInfoBoxHeadlinerColor']
                @eventInfoBoxHoverColor = skin['eventInfoBoxHoverColor']
                @eventInfoBoxLinkColor = skin['eventInfoBoxLinkColor']
                @eventInfoBoxTextColor = skin['eventInfoBoxTextColor']
                @eventInfoSubheadColor = skin['eventInfoSubheadColor']
                @eventInfoSubheadTextColor = skin['eventInfoSubheadTextColor']
                @footerHtml = skin['footerHtml']
                @footerImage = skin['footerImage']
                @footerTextColor = skin['footerTextColor']
                @footerTicketflyLogo = skin['footerTicketflyLogo']
                @headerHtml = skin['headerHtml']
                @headerImage = skin['headerImage']
                @headerTextColor = skin['headerTextColor']
                @headerTicketflyLogo = skin['headerTicketflyLogo']
                @id = skin['id']
                @lastUpdated = skin['lastUpdated']
                @name = skin['name']
                @org = skin['org']
                @secondaryBorderColor = skin['secondaryBorderColor']
                @theme = skin['theme']
                @ticketButtonColor = skin['ticketButtonColor']
                @ticketButtonTextColor = skin['ticketButtonTextColor']
            end
        end
    end
end
