module TicketflyPlus
    class Requestors
        require 'json'
        require 'net/https'
        require 'uri'

        class Base
            BASE_STRING = 'http://www.ticketfly.com/api'

            def initialize(options={})
            end

            def self.request(string)
                url = URI.parse(BASE_STRING + string)
                request = Net::HTTP::Get.new(url.request_uri)
                http = Net::HTTP.new(url.host, url.port)
                result = http.start do |ht|
                    ht.request(request)
                end
                response = JSON.parse(result.body)
            end

            def self.next_page_request(string)
                url = URI.parse(string)
                request = Net::HTTP::Get.new(url.request_uri)
                http = Net::HTTP.new(url.host, url.port)
                result = http.start do |ht|
                    ht.request(request)
                end
                response = JSON.parse(result.body)
            end

            private

            def get_params_list
                self.instance_variables.map {|var| self.instance_variable_get(var)}
            end

            def stringify(prefix, value)
                value != nil ? ("&" + prefix + fill_spaces(value)) : ''
            end

            def fill_spaces(value)
                str = value.to_s.split
                count = 0
                arr = Array.new
                str.each do |word|
                    if count == 0
                        arr.push(word)
                    else
                        arr.push("%20" + word)
                    end
                    count +=1
                end
                arr.join
            end
        end

        class Events < Requestors::Base
            EVENTS_STRING =	'/events'
            attr_accessor :artistName
            attr_accessor :fieldGroup
            attr_accessor :fields
            attr_accessor :fromDate
            attr_accessor :maxResults
            attr_accessor :orgId
            attr_accessor :pageNum
            attr_accessor :q
            attr_accessor :skin
            attr_accessor :tflyTicketed
            attr_accessor :thruDate
            attr_accessor :venueId

            attr_accessor :geo
            attr_accessor :ip
            attr_accessor :clientip
            attr_accessor :distance

            def initialize(options={})
                super
                @artistName = stringify('artistName=', options[:artistName])
                @fields = stringify('fields=', options[:fields])
                @fieldsGroup = stringify('fieldsGroup=', options[:fieldsGroup])
                @fromDate = stringify('fromDate=', options[:fromDate])
                @maxResults = stringify('maxResults=', options[:maxResults])
                @orgId = stringify('orgId=', options[:orgId])
                @pageNum = stringify('pageNum=', options[:pageNum])
                @q = stringify('q=', options[:q])
                @skin = stringify('skin=', options[:skin])
                @tflyTicketed = stringify('tflyTicketed=', options[:tflyTicketed])
                @thruDate = stringify('thruDate=', options[:thruDate])
                @venueId = stringify('venueId=', options[:venueId])
                @geo = stringify('location=geo:', options[:geo])
                @ip = stringify('location=ip:', options[:ip])
                @clientip = stringify('location=clientip', options[:clientip])
                @distance = stringify('distance=', options[:distance])
            end

            def list
                TicketflyPlus::Calls::List.new(requestor_string: EVENTS_STRING, params_list: get_params_list.join.to_s)
            end

            def upcoming
                TicketflyPlus::Calls::Upcoming.new(requestor_string: EVENTS_STRING, params_list: get_params_list.join.to_s)
            end

            def featured
                TicketflyPlus::Calls::Featured.new(requestor_string: EVENTS_STRING, params_list: get_params_list.join.to_s)
            end

            def just_announced
                TicketflyPlus::Calls::JustAnnounced.new(requestor_string: EVENTS_STRING, params_list: get_params_list.join.to_s)
            end

            def past
                TicketflyPlus::Calls::Past.new(requestor_string: EVENTS_STRING, params_list: get_params_list.join.to_s)
            end

            def on_sale
                TicketflyPlus::Calls::OnSale.new(requestor_string: EVENTS_STRING, params_list: get_params_list.join.to_s)
            end

        end

        class Event < Requestors::Base
            EVENT_STRING = '/events'
            attr_accessor :eventId
            attr_accessor :fieldGroup
            attr_accessor :fields
            attr_accessor :showRelatedEvents
            attr_accessor :skin

            def initialize(options = {})
                super
                @eventId = stringify('eventId=', options[:eventId])
                @fields = stringify('fields=', options[:fields])
                @fieldsGroup = stringify('fieldsGroup=', options[:fieldsGroup])
                @showRelatedEvents = stringify('showRelatedEvents=', options[:showRelatedEvents])
                @skin = stringify('skin=', options[:skin])
            end

            def event
                TicketflyPlus::Calls::List.new(requestor_string: EVENT_STRING, params_list: get_params_list.join.to_s)
            end

        end

        class Venues < Requestors::Base
            VENUES_STRING = '/venues'
            attr_accessor :maxResults
            attr_accessor :orgId
            attr_accessor :pageNum
            attr_accessor :venueId

            def initialize(options={})
                super
                @maxResults = stringify('maxResults=', options[:maxResults])
                @orgId = stringify('orgId=', options[:orgId])
                @pageNum = stringify('pageNum=', options[:pageNum])
                @venueId = stringify('venueId=', options[:venueId])
            end

            def list
                TicketflyPlus::Calls::List.new(requestor_string: VENUES_STRING, params_list: get_params_list.join.to_s)
            end
        end

        class Orgs < Requestors::Base
            ORGS_STRING = '/orgs'
            attr_accessor :maxResults
            attr_accessor :orgId
            attr_accessor :pageNum

            def initialize(options={})
                super
                @maxResults = stringify('maxResults=', options[:maxResults])
                @orgId = stringify('orgId=', options[:orgId])
                @pageNum = stringify('pageNum=', options[:pageNum])
            end

            def list
                TicketflyPlus::Calls::List.new(requestor_string: ORGS_STRING, params_list: get_params_list.join.to_s)
            end
        end
    end
end
