module TicketflyPlus
    class Responses
        require 'date'
        require 'time'

        class Base
            attr_accessor :link
            attr_accessor :maxResults
            attr_accessor :pageNum
            attr_accessor :status
            attr_accessor :title
            attr_accessor :totalPages
            attr_accessor :totalResults

            def initialize(response)
                @link = response['link']	
                @maxResults = check_nil(response['maxResults']).to_i 
                @pageNum = check_nil(response['pageNum']).to_i 
                @status = response['status'] 
                @title = response['title']
                @totalPages = check_nil(response['totalPages']).to_i 
                @totalResults = check_nil(response['totalResults']).to_i 
            end

            private

            def check_nil(response)
                !response.nil? ? response : nil
            end
        end

        class Events < Responses::Base
            attr_accessor :events
            attr_accessor :org

            def initialize(response)
                super
                @events = !check_nil(response['events']).nil? ? get_events(response['events']) : response['events']
                @org = !response['org'].nil? ? TicketflyPlus::Objects::Org.new(response['org']) : response['org']
            end

            def get_events(events)
                arr = Array.new
                if events.length > 0
                    events.each do |event|
                        arr.push TicketflyPlus::Objects::Event.new(event)
                    end
                end
                arr
            end
        end

        class Orgs < Responses::Base
            attr_accessor :orgs

            def initialize
                super
                @orgs = !check_nil(response['orgs']).nil? ? get_orgs(response['orgs']) : response['orgs']
            end

            def get_orgs(orgs)
                arr = Array.new
                if orgs.length > 0
                    orgs.each do |org|
                        arr.push TicketflyPlus::Objects::Org.new(org)
                    end
                end
                arr
            end
        end

        class Venues < Responses::Base
            attr_accessor :venues

            def initialize
                super
                @venues = !check_nil(response['venues']).nil? ? get_venues(response['venues']) : response['venues']
            end

            def get_venues(venues)
                arr = Array.new
                if venues.length > 0 
                    venues.each do |venue|
                        arr.push TicketflyPlus::Objects::Venue.new(venue)
                    end
                end
                arr
            end
        end
    end
end
